<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Doctrine\ORM\EntityManager;


class ManutencaoCaixaController extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceName = 'vnd00300_manutencaolancamentocaixa';
    protected $environment;
    protected $novoCodigo;
    protected $ManutencaoVendaService;
    
    public function __construct($crudService, $environment, EntityManager $entityManager, $novoCodigo, $ManutencaoCaixaService, Utilities\Util $utilService) {
        $this->crudService =$crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->novoCodigo  = $novoCodigo;
        $this->ManutencaoCaixaService = $ManutencaoCaixaService;
        $this->utilService = $utilService;
        
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        
        $conditions = $this->utilService->getParamsAsArray($request);
        if( array_key_exists('ATUALIZA_SALDO', $conditions) ) {
            unset($conditions['ATUALIZA_SALDO']);
        }
        $dataSourceFilter = array();
        
        foreach($conditions as $key => $value) {
            if($key != 'LANCAMENTO'){
                $dataSourceFilter[] = $condition = array(
                    "columnName" => $key,
                    "operator"=> "=",
                    "value"=> $value
                );
            }
        }
        
        $original = $request->getFilterCriteria()->getConditions();
            
        for($i = 0 ; $i < count($dataSourceFilter) ; $i++) {
            for($j = 0 ; $j < count($original) ; $j++ ) {
                if( $dataSourceFilter[$i]['columnName'] === $original[$j]['columnName'] &&
                    $dataSourceFilter[$i]['operator'] !== $original[$j]['operator'])  {
                    $dataSourceFilter[$i]['operator'] = $original[$j]['operator'];
                }
                
            }
        }
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        if (count($dataSourceFilter) > 1) { 
            parent::find($newRequest, $response);
        }
    }
    
    public function findLancamentos(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00300_manutencaolancamentocaixa_lancamentos';
        $dataSourceFilter = $request->getFilterCriteria()->getConditions();
        if (!empty($dataSourceFilter[4])) {
            $dataSourceFilter[4]['operator'] = 'IN';
        }
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $this->find($newRequest,$response);
    }
    
    public function findSaldo(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00300_manutencaolancamentocaixa_saldo' ;
        $this->find($request,$response);
    }
    
    public function saveLancamento(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00300_manutencaolancamentocaixa_lancamentos';
        $row = $request->getDataSet()->getRows();
        $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
       
        try {
            foreach($request->getDataSet()->getRows() as $key => &$row) {
            
                $isNew              = $row['__is_new']; 
                $row['NRORG']       = $this->environment->getNrOrgTrab();
               
                if (gettype($row['VRMOVIVEND']) == 'string') {
                    $row['VRMOVIVEND']  = $this->moneyToDouble($row['VRMOVIVEND']);
                }
                $row['CDOPERADOR']  = $this->environment->getCdOperador();
                if (gettype($row['VRMOVIVEOUT']) == 'string') {
                    $row['VRMOVIVEOUT'] = $this->moneyToDouble($row['VRMOVIVEOUT']);
                }
                if(empty($row['NRSEQVENDA']))  
                    $row['NRSEQVENDA'] = null;
                if (empty($row['CDNSUHOSTTEF']))
                    $row['CDNSUHOSTTEF'] = null;
                if(empty($row['NRENVELOPE']))
                    $row['NRENVELOPE'] = null;
                if(empty($row['NRENVELOPE']))
                    $row['NRENVELOPE'] = null;
                if(empty($row['VRCOTINDOUT']))
                    $row['VRCOTINDOUT'] = 0;
                    
                if ($isNew) {
                    //$TDHRABER = explode(' ', $row['DTABERCAIX']);
                    
                    $row['NRSEQUMOVI'] = $this->ManutencaoCaixaService->getNrSequenciaMovimentacao('MOVCAIXA'.$row['CDFILIAL'].$row['CDCAIXA'].$row['DTABERCAIX'], $row['NRORG']);
                    $row['DTHRINCMOV'] = date('d/m/Y H:i:s');
                }   
                
                if (!empty($row['NRSEQVENDA'])) {
                    $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'DTABERCAIX', 'NRSEQVENDA'], $row);
                    if ($this->ManutencaoCaixaService->nrSeqVendaInvalido($parametros)) {
                        throw new \Exception('Operação bloqueada. O número da venda informado não existe para essa abertura.');
                    }
                }
            }
        
            parent::save($request,$response);
            
            if($response->getError() != null){
                $response->addNotification(new Notification($response->getError()->getMessage(), Notification::TYPE_ERROR));
            } else {
    			$return = array(
    			    'NRSEQUMOVI' => $row['NRSEQUMOVI'],
    			    'DTHRINCMOV' => $row['DTHRINCMOV']
    			);
                $response->addDataSet(new DataSet($this->dataSourceName, $return));
                $response->addNotification(new Notification("Lançamento adicionado com sucesso!", Notification::TYPE_SUCCESS));
            }
            
        } catch(\Exception $e){
            $response->addDataSet(new DataSet('error', [$e->getMessage()]));
            // $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function isReducaoZApproved(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00300_manutencaolancamentocaixa_ReducaoZ' ;
        $this->find($request,$response);
    }
    
    public function getIdResEntrada(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'vnd00300_manutencaolancamentocaixa_idResEntrada' ;
        $this->find($request,$response);
    }
    
    public function moneyToDouble($value){
		if ($value){
			$value = preg_replace("/[^0-9,]/", "", $value);
			$value = str_replace ("," , "." , $value);			
		}
		else {
			$value = 0;
		}
		
		return floatval($value);
	}
	
	public function verifyDateParam(DTO\Request\Row $request,DTO\Response $response){
	    $param = $request->getRow();
	    $resultado = $this->ManutencaoCaixaService->verifyDateParam($param['CDFILIAL'],$param['NRORG']);
	    $response->addDataSet(new DataSet("getDateParam", $resultado));
	    
	}
	
	public function atualizaSaldoCaixa(DTO\Request\Row $request,DTO\Response $response){
	    $param = $request->getRow();
	    $resultado = $this->ManutencaoCaixaService->atualizaSaldoCaixa($param['CDFILIAL'],$param['CDCAIXA'],$param['DTMOVTURCAIX'],$param['CDOPERADOR'],$param['DTATUSALDCXA'],$param['NRORG']);
	    $response->addDataSet(new DataSet("atualizaSaldoCaixa", $resultado));
	}
	
	public function TransfereMovimentacao(DTO\Request\DataSet $request, DTO\Response $response){
	    $conditions = $request->getDataSet()->getRows()[0];
	    $resultDataProc = $this->ManutencaoCaixaService->checkDataProc($conditions['CDFILIAL'], $conditions['NRORG'], $conditions['DTABERCAIX'] );
	    if(!$resultDataProc){
	        $response->addMessage(new Message("Operação não permitida. A data de abertura não se enquadra nos parâmetros definidos para a data de processamento.", Message::TYPE_ERROR));
	    } else if(!$conditions['CDCAIXACONT']){
	        $response->addMessage(new Message("Caixa de destino não parametrizado.", Message::TYPE_ERROR));
	    } else {
	        $MOVCAIXA = $this->ManutencaoCaixaService->movCaixa($conditions['CDFILIAL'], $conditions['NRORG'], $conditions['DTABERCAIX'], $conditions['CDCAIXA'] );
	        if(empty($MOVCAIXA)){
	            $response->addMessage(new Message("Não existe movimentação a ser transferida.", Message::TYPE_ERROR));
	        } else {
	            $TURCAIXA = $this->ManutencaoCaixaService->verifTurCaixaAux($conditions['CDFILIAL'], $conditions['NRORG'], $conditions['DTABERCAIX'], $conditions['CDCAIXA'] );
	            if(!empty($TURCAIXA)){
	               $response->addMessage(new Message("Transferência de movimentação para este caixa/abertura já realizada.", Message::TYPE_ERROR)); 
	            } else {
	                $retorno = $this->ManutencaoCaixaService->transferencia($conditions['CDFILIAL'], $conditions['NRORG'], $conditions['DTABERCAIX'], $conditions['CDCAIXA'], $MOVCAIXA, $conditions['CDCAIXACONT'], $conditions['CDOPERADOR'], $conditions['DTMOVTURCAIX']);
	                if($retorno){
	                   $response->addMessage(new Message("Transferência de movimentação realizada com sucesso.")); 
	                }
	            }
	        }
	        
	    }
	    
	}
	
	public function cancelTransf(DTO\Request\Row $request, DTO\Response $response){
	    $param = $request->getRow();
	    $resultado = $this->ManutencaoCaixaService->cancelaTransferencia($param['CDFILIAL'],$param['CDCAIXACONT'],$param['DTABERCAIX'], $param['NRORG'], $param['CDCAIXA']);
	    $response->addMessage(new Message("Transferência de movimentação cancelada com sucesso.")); 
	}
	
	public function auxFind(DTO\Request\Filter $request, DTO\Response $response) {}
	
	public function saveManutencao(DTO\Request\DataSet $request, DTO\Response $response) {
	    $row = $request->getDataSet()->getRows()[0];
        $irregularidades = 0;
	    $informacoes = [];
	    
	    if (!empty($row['FINALIZARAJUSTE']) && $row['FINALIZARAJUSTE']) {
	        $row = $request->getDataSet()->getRows()[0];
	       
	        
	        $informacoes[] = "Vendas com valores irregulares";
            $informacoes[] = "Data da operação: ".date("d/m/Y H:i:s");
            $informacoes[] = "Unidade: ".$row['CDFILIAL']. " - ".$row['NMFILIAL'];
            $informacoes[] = "Caixa: ".$row['CDCAIXA']. " - ".$row['NMCAIXA'];
            $informacoes[] = "Operador do caixa: ".$row['LABELOPERADOR'];
            $informacoes[] = "Data de Abertura: ".$row['DTABERCAIX'];
            $informacoes[] = "";
        
	        $parametros = $this->gerarParametros(['CDFILIAL', 'CDCAIXA', 'DTABERCAIX', 'NRORG'], $row);
	        
	        $vendasIrregulares = $this->ManutencaoCaixaService->vendasIrregulares($parametros);
	        
	        foreach ($vendasIrregulares as $venda) {
	            $irregularidades++;
                $informacoes[] = "Total de entradas da venda ".$venda['NRSEQVENDA']." é diferente do valor total da venda.";
                $informacoes[] = "Total da venda: ".$venda['VRITEMVENDA']." - Total das Movimentações: ".$venda['VRMOVCAIXA']." - Diferença: ".$venda['DIF'];
	            $informacoes[] = "";
	        }
	    }
	    
	    if ($irregularidades == 0) {
	        parent::save($request, $response);
	    } else {
	        $response->addDataset(new DataSet('error', ['Foram encontradas irregularidades nos lançamentos.']));
            $response->addDataset(new DataSet('log', $informacoes));
	    }
	    
	}
	
	private function gerarParametros(array $parametros, $objeto) {
        
        $parametrosQuery = array();
        
        foreach ($parametros as $key) {
            if (empty($objeto[$key])) {
                $parametrosQuery[$key] = null;
            } else {
                $parametrosQuery[':'.$key] = $objeto[$key];
            }
        }
        
        return $parametrosQuery;
    }
    
    public function verificarDtProc($parametros) {
        $caixa = [];
        
        $mensagem = "Operação não permitida. A data de abertura não se enquadra nos parâmetros definidos para a data de processamento.";
        
        if ($caixa['IDCONFVENDA'] == '1') {
            $DTABERCAIX = $parametros['DTABERCAIX'];
        } else {
            $DTABERCAIX = $parametros['DTABERCAIX'];
            // $DTABERCAIX = $caixa['DTMOVTURCAIX'];
        }  
        
        $datas = $this->ManutencaoCaixaService->verifyDateParam($parametros['CDFILIAL'], $parametros['NRORG']);
        
        if ($datas['DTPROCVND'] != '') {
            if ($DTABERCAIX > $datas['DTPROCVND']) {
                if ($datas['IDPERLANCPOSDP'] == 'S') {
                    if ($DTABERCAIX > $datas['DTPROCVND'] + $datas['NRDIAPOSLANC']) {
                        throw new \Exception($mensagem);
                    }
                } else {
                    throw new \Exception($mensagem);
                }
            }

            if ($DTABERCAIX < $datas['DTPROCVND']) {
                if ($datas['IDPERLANCANTDP'] == 'S') {
                    if ($DTABERCAIX < $datas['DTPROCVND'] - $datas['NRDIAPOSLANC']) {
                        throw new \Exception($mensagem);
                    }
                } else { 
                    throw new \Exception($mensagem);
                }
            }
        }
         
        
    }
    
    public function cancelarLancamento(DTO\Request\Row $request,DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            if (($row['IDPERMANUTSAN'] == 'N') and ($row['IDTIPOMOVIVE'] == 'G'))
                throw new \Exception('Operação bloqueada. O tipo de recebimento não está parametrizado para realizar manutenção na sangria.'); //MensErro(msgPadrao.Msg('MOD_N_PERM_SAN'), True);
    
            if (($row['IDPERMANUTVEN'] == 'N') and (($row['IDTIPOMOVIVE'] == 'E') or ($row['IDTIPOMOVIVE'] == 'S') or ($row['IDTIPOMOVIVE'] == 'T')))
                throw new \Exception('Operação bloqueada. O tipo de recebimento não está parametrizado para realizar manutenção na entrada/saída.'); //MensErro(msgPadrao.Msg('MOD_N_PERM_VEN'), True);
            
            $parametros = $this->gerarParametros(['DSMOVIVEND', 'CDFILIAL', 'CDCAIXA', 'DTABERCAIX', 'NRSEQUMOVI'], $row);
            $this->ManutencaoCaixaService->executarQuery(Services\ManutencaoCaixaService::ACMCANCELAMOV, $parametros);
            $response->addDataSet(new DataSet('resultado', array(true)));
        } catch (\Exception $e) {
            $response->addDataSet(new DataSet('resultado', array(false)));
            $response->addDataSet(new DataSet('errorMessage', array($e->getMessage())));
        }
    }
    
    const QRY_CAIXA = "
    SELECT CDCAIXA, NMCAIXA, IDCONFVENDA FROM CAIXA
     WHERE (CDFILIAL = :CDFILIAL) AND (CDLOJA = :CDLOJA) 
       AND (NVL(:CDCAIXA, CDCAIXA) = CDCAIXA) ORDER BY CDCAIXA";
       
    
    
    /**
     * ALTERAÇÕES PARA GERAÇÃO DE FINANCEIRO
     */ 
     
    const RECIBO_PAGTO = 'R';
    
    /**
     * Vinculo de Documento ao Lançamento do Caixa
     * 
     * Quando do tipo escolhido for "Recibo de Pagamento" a rotina irá gerar
     * uma Nota Fiscal e uma Conta de Pagamento para a mesma, sendo do tipo
     * "Nota Fiscal de Entrada" o sistema somente irá cadastrar o registro
     * na tabela DOCSANGRIA
     */
    public function saveDocVinculado(DTO\Request\Row $request,DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $documento = $request->getRow();
            
            // Variáveis de Inicialização
            $nrorg       = $documento['NRORG'];
            $cdfilial    = $documento['CDFILIAL'];
            $cdcaixa     = $documento['CDCAIXA'];
            $dtabercaix  = $documento['DTABERCAIX'];
            $nrsequmovi  = $documento['NRSEQUMOVI'];
            $idtpdocsang = $documento['IDTPDOCSANG'];
            $cdfilialnf  = $documento['CDFILIALNF'];
            
            // Recibo de Pagamento
            if($idtpdocsang == self::RECIBO_PAGTO){
                
                // Variáveis de Inicialização
                $cdfilirece   = $documento['CDFILIAL'];
                $cdserieentr  = $documento['CDSERIEENTR'];
                $cdforneced   = $documento['CDFORNECED'];
                $cdtipooper   = $documento['CDTIPOOPER'];
                $dtatuavenpag = $documento['DTHRINCMOV'];
                $cdtpctpagar  = $documento['CDTPCTPAGAR'];
                $vrnotafisc   = $documento['VRNOTAFISC'];
                
                // Gera Nota Fisca de Pagamento
                $nrlanctonf = $this->ManutencaoCaixaService->geraNFPagamento(
                    $nrorg, $cdfilialnf, $cdfilirece, $cdserieentr, $cdforneced,
                    $cdtipooper, $vrnotafisc, $dtatuavenpag, $cdtpctpagar
                );
            }else{
                $nrlanctonf = $documento['NRLANCTONF'];
            }
            
            // Salva Documento Vinculado ao Lançamento do Caixa
            $this->ManutencaoCaixaService->saveDocVinculado(
                $nrorg, $cdfilial, $cdcaixa, $dtabercaix, $nrsequmovi, $idtpdocsang, $cdfilialnf, $nrlanctonf
            );
            
            $connection->commit();
            $response->addDataSet(new DataSet('result', array(true)));
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->addDataSet(new DataSet('result', array(false)));
            $response->addDataSet(new DataSet('error', array($e->getMessage())));
            throw $e;
        }
    }
    
    public function deleteDocVinculado(DTO\Request\Row $request,DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $documento = $request->getRow();
            
            // Variáveis de Inicialização
            $nrorg       = $documento['NRORG'];
            $cdfilial    = $documento['CDFILIAL'];
            $cdcaixa     = $documento['CDCAIXA'];
            $dtabercaix  = $documento['DTABERCAIX'];
            $nrsequmovi  = $documento['NRSEQUMOVI'];
            $idtpdocsang = $documento['IDTPDOCSANG'];
            $cdfilialnf  = $documento['CDFILIALNF'];
            $nrlanctonf  = $documento['NRLANCTONF'];
            
            // Deleta Documento Vinculado ao Lançamento do Caixa
            $this->ManutencaoCaixaService->deleteDocVinculado(
                $nrorg, $cdfilial, $cdcaixa, $dtabercaix, $nrsequmovi, $idtpdocsang, $cdfilialnf, $nrlanctonf
            );
            
            $connection->commit();
            $response->addDataSet(new DataSet('result', array(true)));
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->addDataSet(new DataSet('result', array(false)));
            $response->addDataSet(new DataSet('error', array($e->getMessage())));
            throw $e;
        }
    }
    
    public function generateFinanceiro(DTO\Request\DataSet $request,DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $dataset = $request->getDataSet()->getRows();
            
            // Variáveis de Inicialização
            $caixa       = $dataset[0];
            $lancamento  = $dataset[1];
            $totalizador = $dataset[2];
            $documentos  = $dataset[3];

            // Gera Financeiro dos Documentos Vinculados
            $this->ManutencaoCaixaService->generateFinanceiro(
                $caixa, $lancamento, $totalizador, $documentos
            );
            
            $connection->commit();
            $response->addDataSet(new DataSet('result', array(true)));
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->addDataSet(new DataSet('result', array(false)));
            $response->addDataSet(new DataSet('error', array($e->getMessage())));
            throw $e;
        }
    }
    
    public function cancelFinanceiro(DTO\Request\DataSet $request,DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $dataset = $request->getDataSet()->getRows();
            
            // Variáveis de Inicialização
            $caixa       = $dataset[0];
            $lancamento  = $dataset[1];
            $totalizador = $dataset[2];
            $documentos  = $dataset[3];

            // Cancela Financeiro dos Documentos Vinculados
            $this->ManutencaoCaixaService->cancelFinanceiro(
                $caixa, $lancamento, $totalizador, $documentos
            );
            
            $connection->commit();
            $response->addDataSet(new DataSet('result', array(true)));
        } catch (\Exception $e) {
            $connection->rollBack();
            $response->addDataSet(new DataSet('result', array(false)));
            $response->addDataSet(new DataSet('error', array($e->getMessage())));
            throw $e;
        }
        
    }
}